@@ -3,21 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * 
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -1131,52 +1132,53 @@ OSStatus	BTInsertRecord		(SFCB						*filePtr,
 	switch (err)				// set/replace/insert decision point
 	{
 		case noErr:			err = fsBTDuplicateRecordErr;
-								goto ErrorExit;
+							goto ErrorExit;
 
 		case fsBTRecordNotFoundErr:	break;
 
 		case fsBTEmptyErr:	// if tree empty add 1st leaf node
 
-								if (btreePtr->freeNodes == 0)
-								{
-									err = ExtendBTree (btreePtr, btreePtr->totalNodes + 1);
-									M_ExitOnError (err);
-								}
+			if (btreePtr->freeNodes == 0)
+			{
+				err = ExtendBTree (btreePtr, btreePtr->totalNodes + 1);
+				M_ExitOnError (err);
+			}
 
-								err = AllocateNode (btreePtr, &insertNodeNum);
-								M_ExitOnError (err);
+			err = AllocateNode (btreePtr, &insertNodeNum);
+			M_ExitOnError (err);
 
-								err = GetNewNode (btreePtr, insertNodeNum, &nodeRec);
-								M_ExitOnError (err);
+			err = GetNewNode (btreePtr, insertNodeNum, &nodeRec);
+			M_ExitOnError (err);
 
-								((NodeDescPtr)nodeRec.buffer)->kind		= kBTLeafNode;
-								((NodeDescPtr)nodeRec.buffer)->height	= 1;
+			((NodeDescPtr)nodeRec.buffer)->kind		= kBTLeafNode;
+			((NodeDescPtr)nodeRec.buffer)->height	= 1;
 
-								recordFit = InsertKeyRecord (btreePtr, nodeRec.buffer, 0,
-															 &iterator->key, KeyLength(btreePtr, &iterator->key),
-															 record->bufferAddress, recordLen );
-								if (recordFit != true)
-								{
-									err = fsBTRecordTooLargeErr;
-									goto ErrorExit;
-								}
+			recordFit = InsertKeyRecord (btreePtr, nodeRec.buffer, 0,
+										 &iterator->key, KeyLength(btreePtr, &iterator->key),
+										 record->bufferAddress, recordLen );
+			if (recordFit != true)
+			{
+				err = fsBTRecordTooLargeErr;
+				goto ErrorExit;
+			}
 
-								err = UpdateNode (btreePtr, &nodeRec);
-								M_ExitOnError (err);
+			err = UpdateNode (btreePtr, &nodeRec);
+			M_ExitOnError (err);
 
-								// update BTreeControlBlock
-								btreePtr->treeDepth	 		= 1;
-								btreePtr->rootNode	 		= insertNodeNum;
-								btreePtr->firstLeafNode		= insertNodeNum;
-								btreePtr->lastLeafNode		= insertNodeNum;
-								M_BTreeHeaderDirty (btreePtr);
+			// update BTreeControlBlock
+			btreePtr->treeDepth	 		= 1;
+			btreePtr->rootNode	 		= insertNodeNum;
+			btreePtr->firstLeafNode		= insertNodeNum;
+			btreePtr->lastLeafNode		= insertNodeNum;
+			M_BTreeHeaderDirty (btreePtr);
 
-								goto Success;
+			goto Success;
 
-		default:				goto ErrorExit;
+		default:				
+			goto ErrorExit;
 	}
 
-	if (index > 0)
+	if (index > 0) 
 	{
 		recordFit = InsertKeyRecord (btreePtr, nodeRec.buffer, index,
 										&iterator->key, KeyLength(btreePtr, &iterator->key),
@@ -1232,7 +1234,6 @@ Success:
 	////////////////////////////// Error Exit ///////////////////////////////////
 
 ErrorExit:
-
 	(void) ReleaseNode (btreePtr, &nodeRec);
 
 	iterator->hint.writeCount 	= 0;
@@ -1756,4 +1757,3 @@ OSStatus	BTInvalidateHint	(BTreeIterator				*iterator )
 	return	noErr;
 }
 
-
