@@ -1,22 +1,25 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2003 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
- *
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
- *
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * 
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * 
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
- *
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
+ * 
  * @APPLE_LICENSE_HEADER_END@
  */
 
@@ -57,12 +60,14 @@ InitBlockCache(SVCB *volume)
  *  kGetBlock
  *  kForceReadBlock
  *  kGetEmptyBlock
+ *  kSkipEndianSwap
  */
 OSStatus
 GetVolumeBlock (SVCB *volume, UInt64 blockNum, GetBlockOptions options, BlockDescriptor *block)
 {
 	UInt32  blockSize;
 	SInt64  offset;
+	UInt16  signature;
 	OSStatus result;	
 	Buf_t *   buffer;
 	Cache_t * cache;
@@ -86,6 +91,16 @@ GetVolumeBlock (SVCB *volume, UInt64 blockNum, GetBlockOptions options, BlockDes
 		block->buffer = NULL;
 	}
 	
+	if (!(options & kSkipEndianSwap) && (result == 0)) {
+		HFSMasterDirectoryBlock *mdb;
+
+		mdb = (HFSMasterDirectoryBlock *)block->buffer;
+		signature = SWAP_BE16(mdb->drSigWord);
+		if (signature == kHFSPlusSigWord )
+			SWAP_HFSPLUSVH(block->buffer);
+		else if (signature == kHFSSigWord)
+			SWAP_HFSMDB(block->buffer);
+	}
 	return (result);
 }
 
@@ -95,6 +110,7 @@ GetVolumeBlock (SVCB *volume, UInt64 blockNum, GetBlockOptions options, BlockDes
  *  kForceWriteBlock
  *  kMarkBlockDirty
  *  kTrashBlock
+ *  kSkipEndianSwap
  */
 OSStatus
 ReleaseVolumeBlock (SVCB *volume, BlockDescriptor *block, ReleaseBlockOptions options)
@@ -103,16 +119,28 @@ ReleaseVolumeBlock (SVCB *volume, BlockDescriptor *block, ReleaseBlockOptions op
 	Cache_t * cache;
 	Buf_t *   buffer;
 	int       age;
+	UInt16  signature;
 
 	cache  = (Cache_t *) volume->vcbBlockCache;
 	buffer = (Buf_t *) block->blockHeader;
 	age    = ((options & kTrashBlock) != 0);
 
-	if (options & (kMarkBlockDirty | kForceWriteBlock))
-		result = CacheWrite(cache, buffer, age, 0);
-	else /* not dirty */
-		result = CacheRelease (cache, buffer, age);
+	/*
+	 * Always leave the blocks in the cache in big endian
+	 */
+	if (!(options & kSkipEndianSwap)) {
+		signature = ((HFSMasterDirectoryBlock *)block->buffer)->drSigWord;
+		if (signature == kHFSPlusSigWord)
+			SWAP_HFSPLUSVH(block->buffer);
+		else if (signature == kHFSSigWord)
+			SWAP_HFSMDB(block->buffer);
+	}
 
+	if (options & (kMarkBlockDirty | kForceWriteBlock)) {
+		result = CacheWrite(cache, buffer, age, 0);
+	} else { /* not dirty */
+		result = CacheRelease (cache, buffer, age);
+	}
 	return (result);
 }
 
