@@ -3,21 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * 
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -79,7 +80,7 @@
 #include <nfs/rpcv2.h>
 #include <nfs/nfsproto.h>
 #include <ufs/ufs/ufsmount.h>
-#include <bsd/isofs/cd9660/cd9660_mount.h>	/* XXX need isofs in include */
+#include <isofs/cd9660/cd9660_mount.h>	/* XXX need isofs in include */
 
 #include <net/if.h>
 #include <net/if_types.h>
@@ -103,9 +104,10 @@
 
 #include <stdarg.h>
 
-#define EXPORT_FROM_NETINFO 0
-#define EXPORT_FROM_FILE 1
-static int source = EXPORT_FROM_NETINFO;
+#define EXPORT_FROM_NETINFO 0	/* export list from NetInfo */
+#define EXPORT_FROM_FILE 1	/* export list from file only */
+#define EXPORT_FROM_FILEFIRST 2	/* export list from file first, then NetInfo */
+static int source = EXPORT_FROM_FILEFIRST;
 
 #include <netinfo/ni.h>
 
@@ -299,8 +301,6 @@ main(int argc, char *argv[])
 	SVCXPRT *udptransp, *tcptransp;
 	int c, i;
 
-	source = EXPORT_FROM_NETINFO;
-
 	while ((c = getopt(argc, argv, "dfnr")) != EOF) {
 		switch (c) {
 			case 'd':
@@ -1101,13 +1101,17 @@ get_exportlist()
 				    fsp->f_mntonname);
 		}
 
-	if (source == EXPORT_FROM_NETINFO)
+	if ((exp_file = fopen(exname, "r")) != NULL) {
+		source = EXPORT_FROM_FILE;
+	} else {
+		if (source == EXPORT_FROM_FILE) {
+			log(LOG_ERR, "Can't open %s", exname);
+			exit(2);
+		}
 		ni_exports_open();
-	else if ((exp_file = fopen(exname, "r")) == NULL) {
-		log(LOG_ERR, "Can't open %s", exname);
-		exit(2);
+		source = EXPORT_FROM_NETINFO;
 	}
-
+		
 	/*
 	 * Read in the exports and build the list, calling mount()
 	 * as we go along to push the export rules into the kernel.
