@@ -3,21 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * 
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -183,8 +184,7 @@ union {
 #define	acg	cgun.cg
 
 #ifdef __APPLE__
-struct dinode zino[ MAXPHYSIO / sizeof(struct dinode)];
-#define ZINOSIZE sizeof(zino)
+#define ZINOSIZE ( sizeof(struct dinode) * (MAXPHYSIO / sizeof(struct dinode)) )
 #else
 struct dinode zino[MAXBSIZE / sizeof(struct dinode)];
 #endif /* __APPLE__ */
@@ -796,6 +796,9 @@ initcg(cylno, utime)
 	long i;
 #ifdef __APPLE__
 	daddr_t wtfsbno;
+	char *	bufferPtr = NULL;
+	char * 	alignedBufPtr;
+	int	pageSize;
 	int zbufsize;
 #endif /* __APPLE__ */
 	register struct csum *cs;
@@ -866,32 +869,37 @@ initcg(cylno, utime)
 #endif /* BIG_ENDIAN_INTEL_FS */
 
 #ifdef __APPLE__
+	// make sure our buffer is page aligned (works around write errors on some drivers)
+	pageSize = getpagesize() - 1;
+	bufferPtr = calloc( ZINOSIZE + pageSize, sizeof( char ) );
+	if ( bufferPtr == NULL ) {
+		printf("calloc failed \n");
+		exit(38);
+	}
+	alignedBufPtr = (char *) ( (u_long)(bufferPtr + pageSize) &~ pageSize );
+	
 	/* Coalesce sequential writes for dinode initialization */
 	for (i = 0, zbufsize=0; i < sblock.fs_ipg / INOPF(&sblock); i += sblock.fs_frag)
-	  {
-	    if (zbufsize + sblock.fs_bsize <= ZINOSIZE)
-	      {
-		if (zbufsize == 0)
-		  wtfsbno = fsbtodb(&sblock, cgimin(&sblock, cylno) + i);
-
+	{
+	    if (zbufsize + sblock.fs_bsize <= ZINOSIZE) {
+			if (zbufsize == 0)
+		  		wtfsbno = fsbtodb(&sblock, cgimin(&sblock, cylno) + i);
 	        zbufsize += sblock.fs_bsize;
 
-		if (zbufsize == ZINOSIZE)
-		  {
-		    wtfs(wtfsbno, zbufsize, (char *)zino);
-		    zbufsize=0;
-		  }
-	      }
-	    else if (zbufsize != 0)
-	      {
-		wtfs(wtfsbno, zbufsize, (char *)zino);
-		wtfsbno = fsbtodb(&sblock, cgimin(&sblock, cylno) + i);
-		zbufsize = sblock.fs_bsize;
-	      }
-
-	  }
+			if (zbufsize == ZINOSIZE) {
+		    	wtfs(wtfsbno, zbufsize, alignedBufPtr);
+		    	zbufsize=0;
+		  	}
+		}
+	    else if (zbufsize != 0) {
+			wtfs(wtfsbno, zbufsize, alignedBufPtr);
+			wtfsbno = fsbtodb(&sblock, cgimin(&sblock, cylno) + i);
+			zbufsize = sblock.fs_bsize;
+		}
+	}
 	if (zbufsize)
-	  wtfs(wtfsbno, zbufsize, (char *)zino);
+	  wtfs(wtfsbno, zbufsize, alignedBufPtr);
+	free( bufferPtr );
 #else
 	for (i = 0; i < sblock.fs_ipg / INOPF(&sblock); i += sblock.fs_frag)
 		wtfs(fsbtodb(&sblock, cgimin(&sblock, cylno) + i),
